import pingen2sdk
import json

from io import IOBase
from typing import Any, Mapping, Optional, Dict, List
from typing_extensions import Literal


class Letters(object):
    def __init__(
        self,
        organisation_id: str,
        access_token: str,
        use_staging: bool = False,
    ):
        self.organisation_id = organisation_id
        self.api_requestor = pingen2sdk.APIRequestor(access_token, use_staging)

    def get_details(
        self,
        letter_id: str,
        params: Optional[Mapping[str, Any]] = None,
        supplied_headers: Optional[Mapping[str, str]] = None,
    ) -> pingen2sdk.PingenResponse:
        return self.api_requestor.perform_get_request(
            "/organisations/%s/letters/%s" % (self.organisation_id, letter_id),
            params,
            supplied_headers,
        )

    def get_collection(
        self,
        params: Optional[Mapping[str, Any]] = None,
        supplied_headers: Optional[Mapping[str, str]] = None,
    ) -> pingen2sdk.PingenResponse:
        return self.api_requestor.perform_get_request(
            "/organisations/%s/letters" % self.organisation_id,
            params,
            supplied_headers,
        )

    def upload_and_create(
        self,
        path_to_file: str,
        file_original_name: str,
        address_position: Literal["left", "right"],
        auto_send: False,
        delivery_product: Optional[str] = None,
        print_mode: Optional[str] = None,
        print_spectrum: Optional[str] = None,
        sender_address: Optional[str] = None,
        meta_data: Optional[Dict] = None,
    ) -> pingen2sdk.PingenResponse:
        file_upload = pingen2sdk.FileUpload(self.api_requestor)
        file_url, file_signature = file_upload.request_file_upload()
        file_upload.put_file(path_to_file, file_url)

        return self.create(
            file_url,
            file_signature,
            file_original_name,
            address_position,
            auto_send,
            delivery_product,
            print_mode,
            print_spectrum,
            sender_address,
            meta_data,
        )

    def create(
        self,
        file_url: str,
        file_signature: str,
        file_original_name: str,
        address_position: Literal["left", "right"],
        auto_send: False,
        delivery_product: Optional[str] = None,
        print_mode: Optional[str] = None,
        print_spectrum: Optional[str] = None,
        sender_address: Optional[str] = None,
        meta_data: Optional[Dict] = None,
    ) -> pingen2sdk.PingenResponse:
        attributes = {
            "file_original_name": file_original_name,
            "file_url": file_url,
            "file_url_signature": file_signature,
            "address_position": address_position,
            "auto_send": auto_send,
        }

        if delivery_product is not None:
            attributes["delivery_product"] = delivery_product

        if print_mode is not None:
            attributes["print_mode"] = print_mode

        if print_spectrum is not None:
            attributes["print_spectrum"] = print_spectrum

        if sender_address is not None:
            attributes["sender_address"] = sender_address

        if meta_data is not None:
            attributes["meta_data"] = meta_data

        return self.api_requestor.perform_post_request(
            "/organisations/%s/letters" % self.organisation_id,
            json.dumps({"data": {"type": "letters", "attributes": attributes}}),
        )

    def send(
        self,
        letter_id: str,
        delivery_product: str,
        print_mode: str,
        print_spectrum: str,
    ) -> pingen2sdk.PingenResponse:
        return self.api_requestor.perform_patch_request(
            "/organisations/%s/letters/%s/send" % (self.organisation_id, letter_id),
            json.dumps(
                {
                    "data": {
                        "id": letter_id,
                        "type": "letters",
                        "attributes": {
                            "delivery_product": delivery_product,
                            "print_mode": print_mode,
                            "print_spectrum": print_spectrum,
                        },
                    }
                }
            ),
        )

    def cancel(
        self,
        letter_id: str,
    ) -> pingen2sdk.PingenResponse:
        return self.api_requestor.perform_cancel_request(
            "/organisations/%s/letters/%s/cancel" % (self.organisation_id, letter_id),
        )

    def delete(
        self,
        letter_id: str,
    ) -> pingen2sdk.PingenResponse:
        return self.api_requestor.perform_delete_request(
            "/organisations/%s/letters/%s" % (self.organisation_id, letter_id),
        )

    def edit(
        self,
        letter_id: str,
        paper_types: List[str],
    ) -> pingen2sdk.PingenResponse:
        return self.api_requestor.perform_patch_request(
            "/organisations/%s/letters/%s" % (self.organisation_id, letter_id),
            json.dumps(
                {
                    "data": {
                        "id": letter_id,
                        "type": "letters",
                        "attributes": {
                            "paper_types": paper_types,
                        },
                    }
                }
            ),
        )

    def get_file(
        self,
        letter_id: str,
    ) -> IOBase:
        return self.api_requestor.perform_stream_request(
            "/organisations/%s/letters/%s/file" % (self.organisation_id, letter_id),
        )

    def calculate_price(
        self,
        country: str,
        paper_types: List[str],
        print_mode: str,
        print_spectrum: str,
        delivery_product: str,
    ) -> pingen2sdk.PingenResponse:
        return self.api_requestor.perform_post_request(
            "/organisations/%s/letters/price-calculator" % self.organisation_id,
            json.dumps(
                {
                    "data": {
                        "type": "letter_price_calculator",
                        "attributes": {
                            "country": country,
                            "paper_types": paper_types,
                            "print_mode": print_mode,
                            "print_spectrum": print_spectrum,
                            "delivery_product": delivery_product,
                        },
                    }
                }
            ),
        )



import json
import requests
import pingen2sdk

from pingen2sdk import error
from requests.models import PreparedRequest
from typing import Dict


class OAuth(object):
    @staticmethod
    def _set_client_id(params):
        if "client_id" in params:
            return

        from pingen2sdk import client_id

        if client_id:
            params["client_id"] = client_id
            return

        raise error.AuthenticationError(
            'No client_id provided. (HINT: set your client_id using "pingen2sdk.client_id = <CLIENT-ID>").'
        )

    @staticmethod
    def _set_client_secret(params):
        if "client_secret" in params:
            return

        from pingen2sdk import client_secret

        if client_secret:
            params["client_secret"] = client_secret
            return

        raise error.AuthenticationError(
            'No client_secret provided. (HINT: set your client_secret using "pingen2sdk.client_secret = <CLIENT-SECRET>").'
        )

    @staticmethod
    def authorize_url(use_staging=False, **params) -> str:
        if use_staging is False:
            path = pingen2sdk.auth_production
        else:
            path = pingen2sdk.auth_staging

        OAuth._set_client_id(params)
        if "response_type" not in params:
            params["response_type"] = "code"

        request = PreparedRequest()
        request.prepare_url(path, params)

        return request.url

    @staticmethod
    def get_token(use_staging=False, **params) -> Dict:
        if use_staging is False:
            url = pingen2sdk.api_production
        else:
            url = pingen2sdk.api_staging

        OAuth._set_client_id(params)
        OAuth._set_client_secret(params)

        response = requests.post(
            url + "/auth/access-tokens",
            params,
            {"Content-Type: application/x-www-form-urlencoded"},
        )

        return json.loads(response.text)

    @staticmethod
    def get_token_from_implicit(fragment: str) -> Dict:
        params = dict(x.split("=") for x in fragment.split("&"))

        return {
            "access_token": params["access_token"],
            "expires_in": params["expires_in"],
        }



import json


class WebhookEvent(object):
    body: str

    def __init__(self, body: str):
        self.body = body

        if bool(body): # pragma: no cover
            self.data = json.loads(body.decode("utf-8"))



import pingen2sdk

from typing import Any, Mapping, Optional


class LetterEvents(object):
    organisation_id: str
    api_requestor: pingen2sdk.APIRequestor

    def __init__(
        self,
        organisation_id: str,
        access_token: str,
        use_staging: bool = False,
    ):
        self.organisation_id = organisation_id
        self.api_requestor = pingen2sdk.APIRequestor(access_token, use_staging)

    def get_collection(
        self,
        letter_id: str,
        params: Optional[Mapping[str, Any]] = None,
        supplied_headers: Optional[Mapping[str, str]] = None,
    ) -> pingen2sdk.PingenResponse:
        return self.api_requestor.perform_get_request(
            "/organisations/%s/letters/%s/events" % (self.organisation_id, letter_id),
            params,
            supplied_headers,
        )

    def get_issue_collection(
        self,
        params: Optional[Mapping[str, Any]] = None,
        supplied_headers: Optional[Mapping[str, str]] = None,
    ) -> pingen2sdk.PingenResponse:
        return self.api_requestor.perform_get_request(
            "/organisations/%s/letters/issues" % self.organisation_id,
            params,
            supplied_headers,
        )



import pingen2sdk
import pytest


class TestIncomingWebhook(object):
    def test_positive(self):
        webhook = pingen2sdk.IncomingWebhook.construct_event(
            b'{"data":{"type":"webhook_issues","id":"309a31e0-1abe-4034-8e7e-1fd473a802fd","attributes":{"reason":"Page limit exceeded","url":"https:\\/\\/5f2e-5-173-206-46.ngrok-free.app\\/webhook","created_at":"2024-04-26T09:54:38+0200"},"relationships":{"organisation":{"links":{"related":"https:\\/\\/api-integration.pingen.com\\/organisations\\/b85c7b52-debb-4b15-b5db-d86b0a4e9bbf"},"data":{"type":"organisations","id":"b85c7b52-debb-4b15-b5db-d86b0a4e9bbf"}},"letter":{"links":{"related":"https:\\/\\/api-integration.pingen.com\\/organisations\\/b85c7b52-debb-4b15-b5db-d86b0a4e9bbf\\/letters\\/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx"},"data":{"type":"letters","id":"xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx"}},"event":{"data":{"type":"letters_events","id":"xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx"}}}},"included":[{"type":"organisations","id":"b85c7b52-debb-4b15-b5db-d86b0a4e9bbf","attributes":{"name":"FreshCompany","status":"active","plan":"free","billing_mode":"prepaid","billing_currency":"CHF","billing_balance":569.72,"missing_credits":0,"edition":"pingen","default_country":"DE","default_address_position":"left","default_sender_address":"Test","data_retention_addresses":6,"data_retention_pdf":1,"limits_monthly_letters_count":null,"color":"#8B27F0","flags":["legitimate","letters_submitted"],"created_at":"2022-01-20T09:00:02+0100","updated_at":"2024-02-28T08:11:46+0100"},"links":{"self":"https:\\/\\/api-integration.pingen.com\\/organisations\\/b85c7b52-debb-4b15-b5db-d86b0a4e9bbf"}},{"type":"letters","id":"xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx","attributes":{"status":"invalid","file_original_name":"example.pdf","file_pages":1,"address":"Example\\nExampleStreet 8\\n3000 City","address_position":"right","country":"CH","delivery_product":"postag_b","print_mode":"simplex","print_spectrum":"color","price_currency":"CHF","price_value":1.63,"paper_types":["normal"],"fonts":[{"name":"ArialMT","is_embedded":true}],"source":null,"tracking_number":null,"sender_address":null,"submitted_at":null,"created_at":"2024-04-26T09:54:38+0200","updated_at":"2024-04-26T09:54:38+0200"},"links":{"self":"https:\\/\\/api-integration.pingen.com\\/organisations\\/b85c7b52-debb-4b15-b5db-d86b0a4e9bbf\\/letters\\/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx"}},{"type":"letters_events","id":"xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx","attributes":{"code":"file_too_many_pages","name":"Page limit exceeded","producer":"Pingen","location":"","has_image":false,"data":[],"emitted_at":"2024-04-26T09:54:38+0200","created_at":"2024-04-26T09:54:38+0200","updated_at":"2024-04-26T09:54:38+0200"}}]}',
            {
                "Content-Type": "application/vnd.api+json",
                "Signature": "36ce106d7bc8c59c039e57d21c127f6942bc0af33bb94c85594cc05b769354a2",
            },
            "webhook_test_secret123",
        )

        assert webhook.data["data"]["type"] == "webhook_issues"

    def test_missing_signature_header(self):
        with pytest.raises(pingen2sdk.error.WebhookSignatureException):
            pingen2sdk.IncomingWebhook.construct_event(
                b'{"data":{"type":"webhook_issues","id":"309a31e0-1abe-4034-8e7e-1fd473a802fd","attributes":{"reason":"Page limit exceeded","url":"https:\\/\\/5f2e-5-173-206-46.ngrok-free.app\\/webhook","created_at":"2024-04-26T09:54:38+0200"},"relationships":{"organisation":{"links":{"related":"https:\\/\\/api-integration.pingen.com\\/organisations\\/b85c7b52-debb-4b15-b5db-d86b0a4e9bbf"},"data":{"type":"organisations","id":"b85c7b52-debb-4b15-b5db-d86b0a4e9bbf"}},"letter":{"links":{"related":"https:\\/\\/api-integration.pingen.com\\/organisations\\/b85c7b52-debb-4b15-b5db-d86b0a4e9bbf\\/letters\\/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx"},"data":{"type":"letters","id":"xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx"}},"event":{"data":{"type":"letters_events","id":"xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx"}}}},"included":[{"type":"organisations","id":"b85c7b52-debb-4b15-b5db-d86b0a4e9bbf","attributes":{"name":"FreshCompany","status":"active","plan":"free","billing_mode":"prepaid","billing_currency":"CHF","billing_balance":569.72,"missing_credits":0,"edition":"pingen","default_country":"DE","default_address_position":"left","default_sender_address":"Test","data_retention_addresses":6,"data_retention_pdf":1,"limits_monthly_letters_count":null,"color":"#8B27F0","flags":["legitimate","letters_submitted"],"created_at":"2022-01-20T09:00:02+0100","updated_at":"2024-02-28T08:11:46+0100"},"links":{"self":"https:\\/\\/api-integration.pingen.com\\/organisations\\/b85c7b52-debb-4b15-b5db-d86b0a4e9bbf"}},{"type":"letters","id":"xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx","attributes":{"status":"invalid","file_original_name":"example.pdf","file_pages":1,"address":"Example\\nExampleStreet 8\\n3000 City","address_position":"right","country":"CH","delivery_product":"postag_b","print_mode":"simplex","print_spectrum":"color","price_currency":"CHF","price_value":1.63,"paper_types":["normal"],"fonts":[{"name":"ArialMT","is_embedded":true}],"source":null,"tracking_number":null,"sender_address":null,"submitted_at":null,"created_at":"2024-04-26T09:54:38+0200","updated_at":"2024-04-26T09:54:38+0200"},"links":{"self":"https:\\/\\/api-integration.pingen.com\\/organisations\\/b85c7b52-debb-4b15-b5db-d86b0a4e9bbf\\/letters\\/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx"}},{"type":"letters_events","id":"xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx","attributes":{"code":"file_too_many_pages","name":"Page limit exceeded","producer":"Pingen","location":"","has_image":false,"data":[],"emitted_at":"2024-04-26T09:54:38+0200","created_at":"2024-04-26T09:54:38+0200","updated_at":"2024-04-26T09:54:38+0200"}}]}',
                {
                    "Content-Type": "application/vnd.api+json",
                },
                "webhook_test_secret",
            )

    def test_webhook_signature_matching_failed(self):
        with pytest.raises(pingen2sdk.error.WebhookSignatureException):
            pingen2sdk.IncomingWebhook.construct_event(
                b'{"data":{"type":"webhook_issues","id":"309a31e0-1abe-4034-8e7e-1fd473a802fd","attributes":{"reason":"Page limit exceeded","url":"https:\\/\\/5f2e-5-173-206-46.ngrok-free.app\\/webhook","created_at":"2024-04-26T09:54:38+0200"},"relationships":{"organisation":{"links":{"related":"https:\\/\\/api-integration.pingen.com\\/organisations\\/b85c7b52-debb-4b15-b5db-d86b0a4e9bbf"},"data":{"type":"organisations","id":"b85c7b52-debb-4b15-b5db-d86b0a4e9bbf"}},"letter":{"links":{"related":"https:\\/\\/api-integration.pingen.com\\/organisations\\/b85c7b52-debb-4b15-b5db-d86b0a4e9bbf\\/letters\\/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx"},"data":{"type":"letters","id":"xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx"}},"event":{"data":{"type":"letters_events","id":"xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx"}}}},"included":[{"type":"organisations","id":"b85c7b52-debb-4b15-b5db-d86b0a4e9bbf","attributes":{"name":"FreshCompany","status":"active","plan":"free","billing_mode":"prepaid","billing_currency":"CHF","billing_balance":569.72,"missing_credits":0,"edition":"pingen","default_country":"DE","default_address_position":"left","default_sender_address":"Test","data_retention_addresses":6,"data_retention_pdf":1,"limits_monthly_letters_count":null,"color":"#8B27F0","flags":["legitimate","letters_submitted"],"created_at":"2022-01-20T09:00:02+0100","updated_at":"2024-02-28T08:11:46+0100"},"links":{"self":"https:\\/\\/api-integration.pingen.com\\/organisations\\/b85c7b52-debb-4b15-b5db-d86b0a4e9bbf"}},{"type":"letters","id":"xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx","attributes":{"status":"invalid","file_original_name":"example.pdf","file_pages":1,"address":"Example\\nExampleStreet 8\\n3000 City","address_position":"right","country":"CH","delivery_product":"postag_b","print_mode":"simplex","print_spectrum":"color","price_currency":"CHF","price_value":1.63,"paper_types":["normal"],"fonts":[{"name":"ArialMT","is_embedded":true}],"source":null,"tracking_number":null,"sender_address":null,"submitted_at":null,"created_at":"2024-04-26T09:54:38+0200","updated_at":"2024-04-26T09:54:38+0200"},"links":{"self":"https:\\/\\/api-integration.pingen.com\\/organisations\\/b85c7b52-debb-4b15-b5db-d86b0a4e9bbf\\/letters\\/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx"}},{"type":"letters_events","id":"xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx","attributes":{"code":"file_too_many_pages","name":"Page limit exceeded","producer":"Pingen","location":"","has_image":false,"data":[],"emitted_at":"2024-04-26T09:54:38+0200","created_at":"2024-04-26T09:54:38+0200","updated_at":"2024-04-26T09:54:38+0200"}}]}',
                {
                    "Content-Type": "application/vnd.api+json",
                    "Signature": "wrong99999999999999999999999999999999999999999999999999signature",
                },
                "webhook_test_secret",
            )



import pingen2sdk
import responses


class TestUsers(object):
    @staticmethod
    def _construct_resource():
        access_token = "test_access_token"

        return pingen2sdk.Users(access_token, True)

    @responses.activate
    def test_get_details(self):
        user_id = "userxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxx1"
        url = "%s/user" % pingen2sdk.api_staging

        users = self._construct_resource()

        responses.get(
            url,
            headers={
                "Content-Type": "application/vnd.api+json",
                "X-Request-Id": "requestx-xxxx-xxxx-xxxx-xxxxxxxxxxx1",
            },
            json={
                "data": {
                    "id": user_id,
                    "type": "users",
                    "attributes": {
                        "email": "email",
                        "first_name": "John",
                        "last_name": "Snow",
                        "status": "active",
                        "language": "en-GB",
                        "edition": "string",
                        "created_at": "2020-11-19T09:42:48+0100",
                        "updated_at": "2020-11-19T09:42:48+0100",
                    },
                    "relationships": {
                        "associations": {
                            "links": {
                                "related": {"href": "string", "meta": {"count": 0}}
                            }
                        },
                        "notifications": {
                            "links": {
                                "related": {"href": "string", "meta": {"count": 0}}
                            }
                        },
                    },
                    "links": {"self": "string"},
                    "meta": {
                        "abilities": {
                            "self": {
                                "reach": "ok",
                                "act": "ok",
                                "resend-activation": "ok",
                            }
                        }
                    },
                },
                "included": [{}],
            },
            status=200,
        )

        response = users.get_details()

        assert response.data["data"]["id"] == user_id
        assert response.data["data"]["type"] == "users"
        assert response.data["data"]["attributes"]["email"] == "email"
        assert response.status_code == 200
